/*
** main.c
**
** Proiect PM
** Audio Spectrum Analyzer
**
** Written by Dragos Diosteanu - 2010
** 
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "lcd.h"
#include "ffft.h"

/* initialize ADC */
void ADC_init(void){
	ADMUX = _BV(REFS0);							// AVCC with external capacitor at AREF pin
	ADCSRA |= _BV(ADPS2)|_BV(ADPS1)|_BV(ADPS0);	// Set division factor to 128
	ADCSRA = _BV(ADEN);							// Enable ADC
}

/*read data from ADC */
int ADC_get(uint8_t chanel){
	ADMUX &= 0xE0;		                 //Save the first 3 bits and set the rest to 0
	ADMUX = chanel;						 //Set the MUXn bits
	ADCSRA |= _BV(ADSC);					 //start ADC
	loop_until_bit_is_set(ADCSRA, ADIF); //loop until the data is ready
	return ADC;
}

/* main -- Program starts here */
int main(void)
{	
	uint8_t channel = PA0;				//the pin where the audio jack is connected
	int i;
	int16_t buffer[FFT_N];				//buffer for captured values type as specified in ffft.h
	complex_t complex_buffer[FFT_N];    //buffer for the Fast Fourier Transformation 
	int16_t result[FFT_N];

	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);
	
	while(1){
		lcd_clear();					//Clear LCD after each pase
		// Print on first line
		lcd_goto_xy(1,1);
		lcd_str("Analizator Spectral 2010");

		// Set starting point
		lcd_goto_xy(1,5);
	
		for (i=0;i<FFT_N;i++)			// Get FFT_N(128 - defined in ffft.h) values
			buffer[i] = ADC_get(channel);
		
		fft_input(buffer,complex_buffer);
		fft_execute(complex_buffer);
		fft_output(complex_buffer, result);

		//draw bars for spectrum
		for (i=0; i<FFT_N/2; i++){
		// The Nokia 3310 lcd has 84x48 pixels. We test this condition for a good display.
			if (result[i]>45)
				result[i]=45;
			draw_bar(i,result[i],0,1);    // x position,height of bar,y position
		}
			
	}
}
